@@ -1,24 +1,26 @@
 #!/usr/bin/python
-# SECUREAUTH LABS. Copyright 2020 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
+#
+# Copyright (C) 2023 Fortra. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author:
-#  Arseniy Sharoglazov <mohemiv@gmail.com> / Positive Technologies (https://www.ptsecurity.com/)
-#
 # Description:
-#  A tool for connecting to MS Exchange via RPC over HTTP v2
+#   A tool for connecting to MS Exchange via RPC over HTTP v2
 #
-# Notes about -rpc-hostname:
-#  Our RPC over HTTP v2 implementation tries to extract the
-#  target's NetBIOS name via NTLMSSP and use it as RPC Server name.
-#  If it fails, you have to manually get the target RPC Server name
-#  from the Autodiscover service and set it in the -rpc-hostname parameter.
+#   Notes about -rpc-hostname:
+#     Our RPC over HTTP v2 implementation tries to extract the
+#     target's NetBIOS name via NTLMSSP and use it as RPC Server name.
+#     If it fails, you have to manually get the target RPC Server name
+#     from the Autodiscover service and set it in the -rpc-hostname parameter.
+#
+# Author:
+#   Arseniy Sharoglazov <mohemiv@gmail.com> / Positive Technologies (https://www.ptsecurity.com/)
 #
 # References:
-#  https://swarm.ptsecurity.com/attacking-ms-exchange-web-interfaces/
+#   - https://swarm.ptsecurity.com/attacking-ms-exchange-web-interfaces/
 #
 
 from __future__ import print_function
@@ -231,6 +233,8 @@ class NSPIAttacks(Exchanger):
         self.__handler = None
 
         self.htable = {}
+        self.anyExistingContainerID = -1
+
         self.props = list()
         self.stat = nspi.STAT()
         self.stat['CodePage'] = nspi.CP_TELETEX
@@ -281,6 +285,20 @@ class NSPIAttacks(Exchanger):
             self.htable[MId]['count'] = self.stat['TotalRecs']
             self.htable[MId]['start_mid'] = self.stat['CurrentRec']
 
+    def load_htable_containerid(self):
+        if self.anyExistingContainerID != -1:
+            return
+
+        if self.htable == {}:
+            self.load_htable()
+
+        for MId in self.htable:
+            self.update_stat(MId)
+
+            if self.stat['CurrentRec'] > 0:
+                self.anyExistingContainerID = NSPIAttacks._int_to_dword(MId)
+                return
+
     def _parse_and_set_htable(self, htable):
         self.htable = {}
 
@@ -442,6 +460,9 @@ class NSPIAttacks(Exchanger):
         printOnlyGUIDs = False
         useAsExplicitTable = False
 
+        if self.anyExistingContainerID == -1:
+            self.load_htable_containerid()
+
         if table_MId == None and eTable == None:
             raise Exception("Wrong arguments!")
         elif table_MId != None and eTable != None:
@@ -529,7 +550,7 @@ class NSPIAttacks(Exchanger):
                     eTableInt = eTable
 
                 resp = nspi.hNspiQueryRows(self.__dce, self.__handler,
-                    ContainerID=0, Count=count, pPropTags=attrs, lpETable=eTableInt)
+                    ContainerID=self.anyExistingContainerID, Count=count, pPropTags=attrs, lpETable=eTableInt)
 
                 try:
                     # Addressing to PropertyRowSet_r must be inside try / except,
@@ -937,7 +958,7 @@ if __name__ == '__main__':
     guid_known.add_argument('-output-file', action='store', help='Output filename')
 
     dnt_lookup = nspi_attacks.add_parser('dnt-lookup', formatter_class=SmartFormatter, help='Lookup Distinguished Name Tags')
-    dnt_lookup.add_argument('-lookup-type', choices=['EXTENDED', 'FULL', 'GUIDS'], nargs='?', default='MINIMAL',
+    dnt_lookup.add_argument('-lookup-type', choices=['EXTENDED', 'FULL', 'GUIDS'], nargs='?', default='EXTENDED',
         help='R|Lookup type:\n'
              '  EXTENDED - Request extended set of fields (default)\n'
              '  FULL     - Request all fields for each row\n'
